@@ -1,6 +1,7 @@
 """CoBib curses interface."""
 
 import curses
+import re
 import sys
 from functools import partial
 from signal import signal, SIGWINCH
@@ -34,6 +35,8 @@ class TUI:
         'top_statusbar': [2, 'black', 'yellow'],
         'bottom_statusbar': [3, 'black', 'yellow'],
         'help': [4, 'white', 'red'],
+        'search_label': [5, 'blue', 'black'],
+        'search_query': [6, 'red', 'black'],
         # TODO when implementing select command add a color configuration option
     }
 
@@ -47,19 +50,14 @@ class TUI:
         'Help': lambda self: self.help(),
         'Open': commands.OpenCommand.tui,
         'Quit': lambda self: self.quit(),
-        'Search': lambda _: None,  # TODO search command
-        'Select': lambda _: None,  # TODO select command
+        'Search': commands.SearchCommand.tui,
+        # TODO select command
+        'Select': lambda self: self.prompt_warning('The Select command is not implemented yet!'),
         'Show': commands.ShowCommand.tui,
         'Sort': partial(commands.ListCommand.tui, sort_mode=True),
         'Wrap': lambda self: self.wrap(),
-        'bottom': lambda self: self.scroll_y('G'),
-        'down': lambda self: self.scroll_y(1),
-        'end': lambda self: self.scroll_x('$'),
-        'home': lambda self: self.scroll_x(0),
-        'left': lambda self: self.scroll_x(-1),
-        'right': lambda self: self.scroll_x(1),
-        'top': lambda self: self.scroll_y(0),
-        'up': lambda self: self.scroll_y(-1),
+        'x': lambda self, update: self.scroll_x(update),
+        'y': lambda self, update: self.scroll_y(update),
     }
 
     # command help strings
@@ -83,24 +81,30 @@ class TUI:
     KEYDICT = {
         10: 'Show',  # line feed = ENTER
         13: 'Show',  # carriage return = ENTER
-        curses.KEY_DOWN: 'down',
-        curses.KEY_LEFT: 'left',
-        curses.KEY_RIGHT: 'right',
-        curses.KEY_UP: 'up',
-        ord('$'): 'end',
+        curses.KEY_DOWN: ('y', 1),
+        curses.KEY_UP: ('y', -1),
+        curses.KEY_NPAGE: ('y', 20),
+        curses.KEY_PPAGE: ('y', -20),
+        ord('j'): ('y', 1),
+        ord('k'): ('y', -1),
+        ord('g'): ('y', 0),
+        ord('G'): ('y', 'G'),
+        2: ('y', -20),  # CTRL-B
+        4: ('y', 10),  # CTRL-D
+        6: ('y', 20),  # CTRL-F
+        21: ('y', -10),  # CTRL-U
+        curses.KEY_LEFT: ('x', -1),
+        curses.KEY_RIGHT: ('x', 1),
+        ord('h'): ('x', -1),
+        ord('l'): ('x', 1),
+        ord('0'): ('x', 0),
+        ord('$'): ('x', '$'),
         ord('/'): 'Search',
-        ord('0'): 'home',
         ord('?'): 'Help',
-        ord('G'): 'bottom',
         ord('a'): 'Add',
         ord('d'): 'Delete',
         ord('e'): 'Edit',
         ord('f'): 'Filter',
-        ord('g'): 'top',
-        ord('h'): 'left',
-        ord('j'): 'down',
-        ord('k'): 'up',
-        ord('l'): 'right',
         ord('o'): 'Open',
         ord('q'): 'Quit',
         ord('s'): 'Sort',
@@ -128,6 +132,7 @@ class TUI:
         self.height, self.width = self.stdscr.getmaxyx()
         self.visible = self.height-3
         # and colors
+        curses.use_default_colors()
         TUI.colors()
         # and user key mappings
         TUI.bind_keys()
@@ -144,6 +149,12 @@ class TUI:
         else:
             self.list_args += ['-r']
 
+        # load further configuration settings
+        if 'TUI' in CONFIG.config.keys():
+            self.prompt_before_quit = CONFIG.config['TUI'].get('prompt_before_quit', True)
+        else:
+            self.prompt_before_quit = True
+
         # Initialize top status bar
         self.topbar = curses.newwin(1, self.width, 0, 0)
         self.topbar.bkgd(' ', curses.color_pair(TUI.COLOR_PAIRS['top_statusbar'][0]))
@@ -206,7 +217,21 @@ class TUI:
     def quit(self):
         """Breaks the key event loop or quits one viewport level."""
         if self.list_mode == -1:
-            raise StopIteration
+            if self.prompt_before_quit:
+                self.prompt.clear()
+                self.prompt.insstr(0, 0, 'Do you really want to quit CoBib? [y/n] ')
+                self.prompt.refresh()
+                key = 0
+                while True:
+                    if key in (ord('y'), ord('Y')):
+                        raise StopIteration
+                    if key in (ord('n'), ord('N')):
+                        break
+                    key = self.prompt.getch()
+                self.prompt.clear()
+                self.prompt.refresh()
+            else:
+                raise StopIteration
         self.update_list()
 
     @staticmethod
@@ -338,21 +363,22 @@ class TUI:
 
         # close help window
         help_win.clear()
+        self.resize_handler(None, None)
 
     def loop(self):
         """The key-handling event loop."""
         key = 0
         # key is the last character pressed
         while True:
-            # reset highlight of current line
-            self.viewport.chgat(self.current_line, 0, curses.A_NORMAL)
-
             # handle possible keys
             try:
                 if key in TUI.KEYDICT.keys():
                     cmd = TUI.KEYDICT[key]
                     if cmd not in self.inactive_commands:
-                        TUI.COMMANDS[cmd](self)
+                        if isinstance(cmd, tuple):
+                            TUI.COMMANDS[cmd[0]](self, cmd[1])
+                        else:
+                            TUI.COMMANDS[cmd](self)
                 elif key == curses.KEY_RESIZE:
                     self.resize_handler(None, None)
             except StopIteration:
@@ -360,6 +386,9 @@ class TUI:
                 break
 
             # highlight current line
+            current_attrs = []
+            for x_pos in range(0, self.width):
+                current_attrs.append(self.viewport.inch(self.current_line, x_pos))
             self.viewport.chgat(self.current_line, 0,
                                 curses.color_pair(TUI.COLOR_PAIRS['cursor_line'][0]))
 
@@ -369,6 +398,10 @@ class TUI:
             # Wait for next input
             key = self.stdscr.getch()
 
+            # reset highlight of current line
+            for x_pos in range(0, self.width):
+                self.viewport.chgat(self.current_line, x_pos, 1, current_attrs[x_pos])
+
     def scroll_y(self, update):
         """Scroll viewport vertically.
 
@@ -384,20 +417,25 @@ class TUI:
             self.top_line = max(self.buffer.height - self.visible, 0)
             self.current_line = self.buffer.height-1
         # scroll up
-        elif update == -1:
+        elif update < 0:
             next_line = self.current_line + update
             if self.top_line > 0 and next_line < self.top_line:
                 self.top_line += update
             if next_line >= 0:
                 self.current_line = next_line
+            else:
+                self.current_line = 0
         # scroll down
-        elif update == 1:
+        elif update > 0:
             next_line = self.current_line + update
-            if next_line - self.top_line == self.visible and \
+            if next_line - self.top_line >= self.visible and \
                     self.top_line + self.visible < self.buffer.height:
                 self.top_line += update
             if next_line < self.buffer.height:
                 self.current_line = next_line
+            else:
+                self.top_line = self.buffer.height - self.visible
+                self.current_line = self.buffer.height - 1
 
     def scroll_x(self, update):
         """Scroll viewport horizontally.
@@ -436,7 +474,8 @@ class TUI:
             out (stream, optional): the output stream to redirect stdout to.
 
         Returns:
-            A list with the executed command to allow further handling.
+            A pair with the first element being the list with the executed command to allow further
+            handling and the second element being whatever is returned by the command.
         """
         # make cursor visible
         curses.curs_set(1)
@@ -490,13 +529,14 @@ class TUI:
         self.prompt.refresh()
 
         # process command if it non empty and actually has arguments
+        result = None
         if command and command[1:]:
             # temporarily disable prints to stdout
             original_stdout = sys.stderr
             sys.stderr = TextBuffer()
             # run command
             subcmd = getattr(commands, command[0].title()+'Command')()
-            subcmd.execute(command[1:], out=out)
+            result = subcmd.execute(command[1:], out=out)
             # if error occurred print info to prompt
             if sys.stderr.lines:
                 self.prompt.addstr(0, 0, sys.stderr.lines[0])
@@ -506,23 +546,37 @@ class TUI:
             # restore stdout
             sys.stderr = original_stdout
         # return command to enable additional handling by function caller
-        return command
+        return (command, result)
+
+    def prompt_warning(self, msg):
+        """Prints a warning to the command prompt.
+
+        Args:
+            msg (str): message text to print.
+        """
+        self.prompt.clear()
+        self.prompt.insstr(0, 0, f'Warning: {msg}')
+        self.prompt.refresh()
 
     def get_current_label(self):
-        """Returns the label of the currently selected entry."""
+        """Returns the label and y position of the currently selected entry."""
+        cur_y, _ = self.viewport.getyx()
         # Two cases are possible: the list and the show mode
         if self.list_mode == -1:
             # In the list mode, the label can be found in the current line
             # or in one of the previous lines if we are on a wrapped line
-            cur_y, _ = self.viewport.getyx()
             while chr(self.viewport.inch(cur_y, 0)) == TextBuffer.INDENT[0]:
                 cur_y -= 1
             label = self.viewport.instr(cur_y, 0).decode('utf-8').split(' ')[0]
-            self.list_mode = cur_y
+        elif re.match(r'\d+ hit',
+                      '-'.join(self.topbar.instr(0, 0).decode('utf-8').split('-')[1:]).strip()):
+            while chr(self.viewport.inch(cur_y, 0)) in ('[', TextBuffer.INDENT[0]):
+                cur_y -= 1
+            label = self.viewport.instr(cur_y, 0).decode('utf-8').split(' ')[0]
         else:
             # In any other mode, the label can be found in the top statusbar
             label = '-'.join(self.topbar.instr(0, 0).decode('utf-8').split('-')[1:]).strip()
-        return label
+        return label, cur_y
 
     def update_list(self):
         """Updates the default list view."""
